// ida.c: S.W. Ellingson, Virginia Tech, 2010 Oct 05
// ---
// COMPILE: (see Makefile)
// ---
// COMMAND LINE SYNTAX, INPUT, OUTPUT: 
//   See help message (search for "Help Message follows:" in source code below)
// ---
// REQUIRES
// ida.h 
// ---
// Immediate data analysis ("IDA") for LWA
// Acquisition, parsing, analysis, and reformatting
// See end of this file for history.

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <time.h>

#include "ida.h"
#include "nr_gasdev.c"
#include "nr_ran1.c"

#define IDA_N_SAMPLES_SIM       12000000 /* number of samples in sim */
#define IDA_OMEGA_TBW        (6.2832e+7) /* = 2*pi*(10e+6) [rad/s] */
#define IDA_OMEGA_TBN        (6.2832e+4) /* = 2*pi*(10e+3) [rad/s] */

/* --- eSource values ---*/
/* positive integers refer to DRs */
#define IDA_FD       -1
#define IDA_FC       -2  /* not yet implemented */
#define IDA_TEST_SIN -3
#define IDA_TEST_WGN -4

/* This could be huge */
static short int xi[IDA_N_SAMPLES_MAX];
static short int xq[IDA_N_SAMPLES_MAX];

main ( int narg, char *argv[] ) {

  /*=================*/
  /*=== Variables ===*/
  /*=================*/
 
  int iarg;

  int eSource = IDA_TEST_SIN;                 /* <source>; set by "-s" switch */
  char src_filename[IDA_MAX_FILENAME_LENGTH]; /* <filename>; set by "-sFD=" switch */
  int nSensor=0;                              /* <sensor>; set by "-n" switch */
  int bVerbose=0;                             /* set by "-v" switch */
  int eMode=IDA_MODE_TBW;                     /* <mode>; set as second parameter in command line */

  int bDone;
  int bDone2;

  int eStatus = -99;    /* return code from tbw_file_info (in idav.h) */
  int b4bit = -99;      /* see ida_parse_file() (in idav.h) */
  long int nFrames = -99;        /* see ida_parse_file (in idav.h) */
  long int nFramesBad = -99;     /* see ida_parse_file (in idav.h) */
  long int iFirstBadFrame = -99; /* see ida_parse_file (in idav.h) */

  int iStandMax = -99;  /* see tbw_file_info (in tbw.h) */
  unsigned long int time0 = -99;  /* see tbw_file_info (in tbw.h) */
  
  unsigned long int time1;

  unsigned short int x0;
  
  int i;
  int iSensor1, iSensor2;
  int iStand;
  int iPol;

  long int k,l;

  long int lseed;

  short int mini, minq, maxi, maxq;
  short int min_code, max_code;
  float meani, meanq, stdi, stdq;
  long int ccpi, ccpq, ccmi, ccmq;

  FILE *fp1;
  FILE *fp2;
  char filename[IDA_MAX_FILENAME_LENGTH];

  /*==================*/
  /*=== Initialize ===*/
  /*==================*/
    
  /* First, announce thyself */
  printf("[%s] I am %s\n",ME,MY_NAME);

  /* clearing this just in case */
  sprintf(src_filename,"");

  /* seed for the random number generator used for IDA_TEST_WGN */
  lseed = (long int) time(NULL); 

  /* process command line arguments */
  for ( iarg=2; iarg<=narg; iarg++) {

    bDone2 = 0;

    /* sense & process "-h" */
    if (!strncmp(argv[iarg-1],"-h",2)) {
      bDone2 = 1;
      /* Help Message follows: */
      printf("COMMAND LINE:\n");
      printf("  ida -h -m<mode> -s<source> -n<sensor> -v -c\n");
      printf("WHAT I DO:\n");
      printf("  Acquire, read, or simulate data from DP via DR.\n");
      printf("  Parse data into separate files per sensor/pol, plus a file for time tags.\n");
      printf("  Compute diagnostic statistics.\n");
      printf("  Pre-processor for the data visualization program idav.\n");
      printf("INPUT:\n");
      printf("  -h means show help, then exit.\n");
      printf("  -m specifies mode.  Options for <mode> are:\n");
      printf("     w (as in '-mw') means assume TBW-mode data. [DEFAULT]\n");
      printf("     n (as in '-mn') means assume TBN-mode data.\n");
      printf("     d (as in '-md') means assume DRX-mode data.\n");
      printf("  -s specifies source of data to analyze.  Options for <source> are:\n");
      printf("     DR#; e.g., DR1 means get a fresh acquisition via DR1.\n");
      printf("       (Must be the one connected to TBW!)\n"); 
      printf("     FD=<filename> means use <filename>; assumed to be from a DR.\n");
      //printf("     FC=<filename> means use <filename>, assumed to be single-\n");
      //printf("       sensor space-delimited ASCII data\n");
      printf("     test_sin means use a single-sensor simple internally-generated test\n");
      printf("       sinusoid. 12,000,000 samples of 12-bit data. [DEFAULT]\n");
      printf("     test_wgn means use a single-sensor simple internally-generated white\n");
      printf("       gaussian noise. 12,000,000 samples of 12-bit data.\n");
      printf("  -n specifies sensor; meaningful only if <source> is DR or FD.\n");
      printf("     <sensor> is in the range 1 - 520 (less if TBW supports less).\n");
      printf("     1 & 2 correspond to Stand 1; 3 & 4 correspond to Stand 2; and so on\n");
      printf("     (This is DP-output numbering; may not correspond to actual stand/pol IDs)\n");
      printf("     <sensor>=0 means process all available sensors [DEFAULT]\n");
      printf("  -v invokes verbose output. Default is not to be verbose.\n");
      printf("  -c (clean) deletes out files from previous run(s) and exits. Default is don't.\n");
      printf("OUTPUT:\n");
      printf("  ida###.dat (where ###=<sensor>): time series (A/D codes), binary format.\n");
      printf("    Intended primary for idav, but format is simple (see ida.h)\n");
      printf("  ida_tt.dat: Table of time tags seen.\n");
      printf("    Columns=stands, rows=frames, decimal in space-delimited ASCII format.\n");
      printf("    Shown are the actual time tags minus the value of first time tag seen.\n");
      printf("    (Value of first time is written as a status message to stdout.)\n");
      printf("  ida_stats.dat: Statistics for <sensor> (or all sensors, depending on -n).\n");
      printf("    ASCII, space-delimited, one sensor per line.  Line format follows:\n");
      printf("    sensor_ID mean std_dev min_code max_code neg_clips pos_clips\n");
      printf("      min/max_code are smallest/largest A/D codes seen.\n");
      printf("      neg/pos_clips are numbers of 0's/4095's seen.\n");
      printf("  ida_t###.dat: Time series for sensor ###, in A/D codes, ASCII.\n");
      return;
      } /* if (!strncmp(argv[iarg-1],"-FD=",4))  */

    /* sense & process "-m" */
    if (!strncmp(argv[iarg-1],"-m",2)) {
      bDone2 = 1;
      if (strlen(argv[iarg-1])<3) {
        printf("[%s] FATAL: Can't parse parameter of recognized switch '%s'\n",ME,argv[iarg-1]);
        return;
        }
      switch (argv[iarg-1][2]) {
        case 'w': eMode = IDA_MODE_TBW; break;
        case 'n': eMode = IDA_MODE_TBN; break;
        case 'd': eMode = IDA_MODE_DRX;
          printf("[%s] FATAL: Concerning switch '%s': DRX mode not implemented.\n",ME,argv[iarg-1]);
          return;
          break;
        default:
          printf("[%s] FATAL: Can't parse parameter of recognized switch '%s'\n",ME,argv[iarg-1]);
          return;
          break;    
        } 
      } /* if (!strncmp(argv[iarg-1],"-m",2))  */

    /* sense & process "-s" */
    //printf("%d\n",strncmp(argv[1],"-s",2));
    if (!strncmp(argv[iarg-1],"-s",2)) {
      bDone2 = 1;
      bDone = 0;
      //printf("'%s'\n",argv[iarg-1]);
      if (!strncmp(argv[iarg-1],"-sDR",4)) {
        bDone = 1;
        if (strlen(argv[iarg-1])<5) {
          printf("[%s] FATAL: Can't parse parameter of recognized switch '%s'\n",ME,argv[iarg-1]);
          return;
          }
        sscanf( &(argv[iarg-1][4]), "%d", &eSource ); 
        //printf("DR '%s' %d\n",&(argv[iarg-1][4]),eSource); 
        }         
      if (!strncmp(argv[iarg-1],"-sFD",4)) {
        bDone = 1;
        //printf("FD\n");
        eSource = IDA_FD; 
        if (strlen(argv[iarg-1])<6) {
          printf("[%s] FATAL: Can't parse parameter of recognized switch '%s'\n",ME,argv[iarg-1]);
          return;
          }
        sscanf( &(argv[iarg-1][5]), "%s", src_filename ); 
        } 
      //if (!strncmp(argv[iarg-1],"-sFC=",4)) {
      //  bDone = 1;
      //  //printf("FC\n"); 
      //  eSource = IDA_FC;
      //  if (strlen(argv[iarg-1])<6) {
      //    printf("[%s] FATAL: Can't parse parameter of recognized switch '%s'\n",ME,argv[iarg-1]);
      //    return;
      //    }
      //  sscanf( &(argv[iarg-1][5]), "%s", src_filename ); 
      //  } 
      if (!strncmp(argv[iarg-1],"-stest_sin",10)) {
        bDone = 1;
        //printf("test_sin\n");
        eSource = IDA_TEST_SIN; 
        } 
      if (!strncmp(argv[iarg-1],"-stest_wgn",10)) {
        bDone = 1;
        // printf("test_wgn\n"); 
        eSource = IDA_TEST_WGN; 
        }
      if (!bDone) {
        printf("[%s] FATAL: Can't parse parameter of recognized switch '%s'\n",ME,argv[iarg-1]);
        return;
        }
      } /* if (!strncmp(argv[iarg-1],"-s",2)) */

    /* sense & process "-n" */
    if (!strncmp(argv[iarg-1],"-n",2)) {
      bDone2 = 1;
      if (strlen(argv[iarg-1])<3) {
        printf("[%s] FATAL: Can't parse parameter of recognized switch '%s'\n",ME,argv[iarg-1]);
        return;
        }
      sscanf( &(argv[iarg-1][2]), "%d", &nSensor );       
      } /* if (!strncmp(argv[iarg-1],"-n",2))  */

    /* sense & process "-v" */
    if (!strncmp(argv[iarg-1],"-v",2)) {
      bDone2 = 1;
      bVerbose = 1;
      } /* if (!strncmp(argv[iarg-1],"-v",2))  */

    /* sense & process "-c" */
    if (!strncmp(argv[iarg-1],"-c",2)) {
      bDone2 = 1;
      system("rm ida???.dat"); /* takes care of ida_tt.dat as well */
      system("rm ida_stats.dat");
      system("rm ida_t???.dat");
      return;
      } /* if (!strncmp(argv[iarg-1],"-c",2))  */

    if (!bDone2) {
      printf("[%s] FATAL: Can't parse switch '%s'\n",ME,argv[iarg-1]);
      return;
      }
 
    } /* for (iarg */

  /* Show what we know about this run... */
  printf("[%s] PARAMETER: eMode = %d\n",ME,eMode);
  printf("[%s] PARAMETER: eSource = %d\n",ME,eSource);
  printf("[%s] PARAMETER: src_filename = '%s'\n",ME,src_filename); 
  printf("[%s] PARAMETER: nSensor = %d\n",ME,nSensor); 
  printf("[%s] PARAMETER: bVerbose = %d\n",ME,bVerbose); 
  //return;

  /*================================================*/
  /*=== Doin' It ===================================*/
  /*================================================*/

  if (eMode==IDA_MODE_TBN) { 
    //printf("[%s] FATAL: TBN mode operation not implemented.\n",ME);  
    //return; 
    }
  if (eSource>0) { /* source is a data recorder.  need to acquire a file. */
    printf("[%s] FATAL: -sDR# option disabled.\n",ME);  
    return; 
    }

  /* At this point, "src_filename" should now be filename of acquired file */

  if ( (eSource>0) || (eSource==IDA_FD) ) { /* source is a previously-acquired file. */

    printf("[%s] Stand by: reading '%s'...\n",ME,src_filename);
    eStatus = ida_parse_file( src_filename, 
                              eMode, 
                              bVerbose, 
                              &time0, 
                              &nFrames,
                              &nFramesBad,
                              &iFirstBadFrame, 
                              &b4bit, 
                              &iStandMax );

    /* at this point ida###.dat files should exist locally */

    printf("[%s] ida_parse_file() complete. Results follow: --------------\n",ME);
    printf("[%s] eStatus = %d (0 is OK, and 2 is probably OK. see ida.h for others)\n",ME,eStatus);
    printf("[%s] nFrames        = %ld (number of 0xDEC0DE5C's seen)\n",ME,nFrames);
    printf("[%s] nFramesBad     = %ld (number of invalid frames seen)\n",ME,nFramesBad);
    if (nFramesBad>0) 
      printf("[%s] iFirstBadFrame = %ld (serial number of first bad frame)\n",ME,iFirstBadFrame);
    printf("[%s] iStandMax = %d (highest stand index seen (1..520; 0 if not seen))\n",ME,iStandMax);
    printf("[%s] time0 = %lu (first time tag seen; -99 if not seen)\n",ME,time0);
    //printf("[%s] %ld snapshots found, corresponding to %f ms\n",ME,(nFrames/iStandMax)*(400+b4bit*800),
    //                                                              ((nFrames/iStandMax)*(400+b4bit*800))/(196.0e+3));
    time1 = time0/IDA_FS_TBW; /* converting units of time from samples (at rate IDA_FS_TBW) to seconds */
    printf("[%s] Time indicated by first time tag: %s",ME,ctime(&time1));
    if (eMode==IDA_MODE_TBW) {
      printf("[%s] b4bit = %d (1 means 4-bit data, 0 means 12-bit data)\n",ME,b4bit);
      }

    /* Decide about how to go forward */
    if ( eStatus != IDA_STATUS_OK)  {
      printf("[%s] WARNING: I have concerns. Check eStatus (above) for more info.\n",ME,src_filename);
      if ( (eStatus & IDA_STATUS_DEC0DE5C_NOT_FOUND) != 0 )
        printf("[%s] ...Extraneous bytes found in file (IDA_STATUS_DEC0DE5C_NOT_FOUND).\n",ME);
      if (nFramesBad>0) 
        printf("[%s] ...Data from invalid frames is not written to files.\n",ME);  
      if ( iFirstBadFrame == 1 ) {
        printf("[%s] FATAL: No valid frames.\n",ME);  
        return; 
        }
      //nFrames = iFirstBadFrame-1;
      //printf("[%s] WARNING: Resetting nFrames to iFirstBadFrame-1 = %ld.\n",ME,nFrames); 
      }

    } /* if ( (eSource>0) || (eSource==IDA_FD) ) */

  //return;

  /* Figure out range of sensor IDs to be considered */
  iSensor1 = nSensor; /* if specific sensor is specified, we do just that sensor */
  iSensor2 = nSensor; 
  if (nSensor==0) {   /* if all sensors are to be done: */
    iSensor1 = 1;
    iSensor2 = 2*iStandMax;
    }
  if (eSource<-2) {  /* then this is simulation (IDA_TEST_SIN or TEST_WGN) */
    iSensor1 = 1; /* this is just a kludge to get into the main processing loop below */
    iSensor2 = 1;
    }

  /* Create mock data and write it to ida001.dat */
  if ( (eSource==IDA_TEST_SIN) || (eSource==IDA_TEST_WGN) ) {
    fp1 = fopen("ida001.dat","wb");
    for (l=0;l<IDA_N_SAMPLES_SIM;l++) {
      if (eSource==IDA_TEST_SIN) {
        if (eMode==IDA_MODE_TBW) {
          xi[l] = (short int) ( 2048 + 2000*sin(IDA_OMEGA_TBW*l/IDA_FS_TBW) );
          fwrite(&(xi[l]),sizeof(short int),1,fp1);
          }
        if (eMode==IDA_MODE_TBN) {
          xi[l] = (short int) (  128 +  100*cos(IDA_OMEGA_TBN*l/IDA_FS_TBN) );
          xq[l] = (short int) (  128 +  100*sin(IDA_OMEGA_TBN*l/IDA_FS_TBN) );
          x0 = ( xi[l] << 8 ) | ( xq[l] & 0x00FF ); /* pack them into a short int */ 
          fwrite(&(x0),sizeof(short int),1,fp1);
          }
        if (eMode==IDA_MODE_DRX) {
          xq[l] = 0;
          xq[l] = 0;
          }
        }
      if (eSource==IDA_TEST_WGN) {
        printf("[%s] FATAL: WGN generator says 'Fix Me!'\n",ME);
        return;
        //xi[l] = (short int) ( 48 + ( rand() % 4000 ) );
        //xi[l] = (short int) ( 2048 + 500*gasdev(&lseed) );
        }
      }
    fclose(fp1);
    b4bit=0; /* has to be set so that clip counter works correctly */
    printf("[%s] Made %ld samples\n",ME,l);
    }

  printf("[%s] Reading/analyzing sensor (ida###.dat) files: ---------\n",ME);

  fp1 = fopen("ida_stats.dat","w");

  /* Loop through sensors */
  for ( i=iSensor1; i<=iSensor2; i++ ) {
    //printf("[%s] Doing Sensor=%d\n",ME,i);

    /* For each sensor considered, we need to get data into memory */
    /* How we do this depends on where the data is currently at */

    if ( (eSource>0) || (eSource==IDA_FD) ) { /* source is a previously-acquired file. */ 
      iStand = (i-1)/2 + 1;
      iPol = i-2*(iStand-1); 
      //printf("[%s] Now looking at iStand=%d, iPol=%d\n",ME,iStand,iPol);  
      if (!(ida_sfile_import( iStand, iPol, eMode, b4bit, xi, xq, &l ))) {
        printf("[%s] FATAL: ida_sfile_import() failed.\n",ME);
        return; 
        }
      printf("[%s] Found %ld samples\n",ME,l);
      } /* if ( (eSource>0) || (eSource==IDA_FD) ) */

    /* At this point, data is in memory regardless of source/format */

    /* Get ready to compute first-pass statistics */
    meani = 0;   /* mean */
    meanq = 0; 
    mini = xi[0]; /* min value */
    maxi = xi[0]; /* max value */
    minq = xq[0]; /* min value */
    maxq = xq[0]; /* max value */
    ccpi = 0;    /* positive clip count */
    ccpq = 0;
    ccmi = 0;    /* negative clip count */
    ccmq = 0;
    min_code = 0;
    switch (eMode) {
      case IDA_MODE_TBW:
        if (b4bit) { max_code = 15; } else { max_code = 4095; }
        break;
      case IDA_MODE_TBN:  
        max_code = 255;
        break;
      case IDA_MODE_DRX:
      default:
        printf("[%s] FATAL: Bogus eMode=%d.\n",ME,eMode);
        break;
      }
    //printf("* max_code=%d\n",max_code);

    /* Compute first-pass statistics */
    for (k=0;k<l;k++) {

      meani += xi[k];
      if (xi[k]<mini) { mini=xi[k]; }
      if (xi[k]>maxi) { maxi=xi[k]; }
      if (xi[k]>=max_code) { ccpi++; /* printf("%d %d\n",x[k],max_code); */ }
      if (xi[k]<=min_code) { ccmi++; }

      if ( (eMode==IDA_MODE_TBN) || (eMode==IDA_MODE_DRX) ) {
        meanq += xq[k];
        if (xq[k]<minq) { minq=xq[k]; }
        if (xq[k]>maxq) { maxq=xq[k]; }
        if (xq[k]>=max_code) { ccpq++; /* printf("%d %d\n",x[k],max_code); */ }
        if (xq[k]<=min_code) { ccmq++; }        
        } /* if ( (eMode==IDA_MODE_TBN) || (eMode==IDA_MODE_DRX) ) */

      } /* for (k=0;k<l;k++) */
    meani /= l;
    meanq /= l;

    /* Compute second-pass statistics */
    stdi = 0;   /* standard deviation */
    stdq = 0;
    for (k=0;k<l;k++) {
      stdi += ( xi[k] - meani ) * ( xi[k] - meani );   /* interim; taking sqrt later */
      if ( (eMode==IDA_MODE_TBN) || (eMode==IDA_MODE_DRX) ) {
        stdq += ( xq[k] - meanq ) * ( xq[k] - meanq ); /* interim; taking sqrt later */
        } /* if ( (eMode==IDA_MODE_TBN) || (eMode==IDA_MODE_DRX) ) */
      } /* for (k=0;k<l;k++) */
    stdi = sqrt(stdi/l);
    stdq = sqrt(stdq/l);

    printf("[%s] %d %f %f %d %d %ld %ld %f %f %d %d %ld %ld\n",ME,
      i,meani,stdi,mini,maxi,ccmi,ccpi,
        meanq,stdq,minq,maxq,ccmq,ccpq);
    fprintf(fp1,"%3d %6.1f %6.1f %4d %4d %10ld %10ld %6.1f %6.1f %4d %4d %10ld %10ld\n",   
      i,meani,stdi,mini,maxi,ccmi,ccpi,
        meanq,stdq,minq,maxq,ccmq,ccpq);

    /* Write a copy of the time-domain waveform */
    sprintf(filename,"ida_t%03d.dat",i);
    fp2 = fopen(filename,"w");
    for (k=0;k<l;k++) fprintf(fp2,"%4d %4d\n",xi[k],xq[k]);
    fclose(fp2);

    } /* for ( i=iSensor1 ... */

  fclose(fp1);

  return;
  } /* main() */

//==================================================================================
//=== HISTORY ======================================================================
//==================================================================================
// ida.c: S.W. Ellingson, Virginia Tech, 2010 Oct 05
//   .1: Initial version, starting with mtbw.c from MTBW v.0.3
// mtbw.c: S.W. Ellingson, Virginia Tech, 2010 Oct 05
//   .1: Attempts to decode time-tag 
// mtbw.c: S.W. Ellingson, Virginia Tech, 2010 Sep 30
//   .1: removed diagnostic printf that shows clipping instances 
//       added -v (verbose) option
//       added -c (clean: remove output files from previous run) option
//       help message discloses the mtbw_t???.dat files
// mtbw.c: S.W. Ellingson, Virginia Tech, 2010 Sep 23
//   .1: initial version


//==================================================================================
//=== BELOW THIS LINE IS SCRATCH ===================================================
//==================================================================================

