// ida.h: S.W. Ellingson, Virginia Tech, 2010 Oct 05
// auxillary source code for ida.c
// See end of this file for history.

#define MY_NAME "ida (v.20101005.1)"
#define ME "16" /* listed in LWA_MCS.h */

#define IDA_MAX_FILENAME_LENGTH 256

#define IDA_FS_TBW         196000000 /* sample rate as an integer (don't make a float) */ 
#define IDA_FS_TBN            100000 /* sample rate as an integer (don't make a float) */

#define IDA_N_SAMPLES_MAX   36000000
#define IDA_N_STANDS_MAX         261 

#define IDA_N_FRAMES_PER_CHANNEL 30001 /* This is the maximum number of frames per polarization in a single trigger */
                                       /* = 12 000 000 samples /  400 samples/packet for 12-bit data */
                                       /* = 36 000 000 samples / 1200 samples/packet for  4-bit data */

#define IDA_MODE_TBW 1
#define IDA_MODE_TBN 2
#define IDA_MODE_DRX 3

/* used as return error codes by ida_parse_file() */
#define IDA_STATUS_OK                 0
#define IDA_STATUS_FILE_NOT_FOUND     1
#define IDA_STATUS_DEC0DE5C_NOT_FOUND 2 /* Means this happened at least once */
#define IDA_STATUS_WRONG_TB           4 /* TBW/TBN bit in TB?_ID doesn't match eMode */
#define IDA_STATUS_4BIT_CHANGED       8 /* TBW_ID bit flag indicating bits/sample changed mid-file */
#define IDA_STATUS_INVALID_MODE      16 /* eMode is not a recognized/implemented value */

/* Storage of remembering time tags for every packet */
static unsigned long int ttt[IDA_N_STANDS_MAX*2][IDA_N_FRAMES_PER_CHANNEL];


/****************************************************************************/
/*** ida_parse_file() *******************************************************/
/****************************************************************************/

int ida_parse_file( 
  char     *filename,       /* IN:  filename of TBW-generated file (via DR) to read/parse */
  int      eMode,           /* IN:  assumed DP mode (TBW, TBN, or DRX).  Should = IDA_MODE_??? */
  int      bVerbose,        /* IN:  =1 means "be verbose" */
  unsigned long int *time0, /* OUT: value of first time tag seen */
  long int *nFrames,        /* OUT: total number of frames seen (counting 0xDEC0DE5C's) */
  long int *nFramesBad,     /* OUT: number of frames that were identified as invalid */
  long int *iFirstBadFrame, /* OUT: serial number of first invalid frame */
  int      *b4bit,          /* OUT: 1 means data is self-identified as 4-bit (as opposed to 12-bit) data */
                            /*      if not eMode != IDA_MODE_TBW, this is meaningless */
  int      *iStandMax       /* OUT: index (1..260) of highest-numbered stand seen in TBW_ID or TBN_ID */ 
                            /*      if not eMode != IDA_MODE_TBW or _TBN, this is meaningless */
  ) {
/* Returns a value IDA_STATUS_* which can be used as an error code */ 
/* TBW and TBN data is parsed and written binary to files ida###.dat where ### is stand number */
  
  int eStatus = IDA_STATUS_OK; /* Assessment of file (until we find otherwise) */                  

  FILE *fp;
  FILE *fpo[2*IDA_N_STANDS_MAX]; /* one file per stand and polarization */

  int b,b1,b2,bb;
  int k;
  char outfilename[IDA_MAX_FILENAME_LENGTH];
  
  int bDEC0DE5C;     /* Have we seen the start-of-packet pattern yet? */
  int b4bit0 = -1;   /* "-1" indicates we have not read the 4/12-bit flag bit of TBW_ID yet */ 
  int bTBWBit;
  int bBadFrame;

  int bStand[IDA_N_STANDS_MAX]; /* =1 means we've seen this stand in the data (TBW/TBN only) */
  int iStand;
  int iChan;
  long int iFrame[IDA_N_STANDS_MAX*2]; /* keeping track of the number frames seen, by stand and pol */
  long int iFrameMax;

  long int l;
  unsigned long int ttt0 = 0;

  union {
    unsigned long int i;
    unsigned char b[8];
    } i8u;

  short int x0, y0;
  char c,c1,c2;
  char x0c,y0c;

  /* initialize variables */
  *time0 = 0;
  *nFrames = 0;
  *nFramesBad = 0;
  *iFirstBadFrame = 0;
  *b4bit = -1;
  *iStandMax = 0;
  for (k=1;k<=IDA_N_STANDS_MAX;k++) { 
    bStand[k]=0;
    iFrame[2*k-1]=0;
    iFrame[2*k  ]=0;
    for (l=0;l<IDA_N_FRAMES_PER_CHANNEL;l++) {
      ttt[2*k-1][l] = 0;
      ttt[2*k  ][l] = 0;
      }
    }
  iStand = 0;

  /* attempt to open file */
  if (!(fp = fopen(filename,"rb"))) {
    eStatus = eStatus | IDA_STATUS_FILE_NOT_FOUND;
    return eStatus; 
    }

  /* for DRX mode, we'll pre-open an output file. (For TBW/TBN, these are opened as stands appear.) */
  if (eMode==IDA_MODE_DRX) {
    fpo[1] = fopen("ida001.dat","wb");
    *iStandMax=1;
    iStand=1;
    }

  /* looping through packets */
  while ( (b = fgetc(fp)) != EOF ) {

    /* Read the 0xDEC0DE5C (4-byte) pattern */
    bDEC0DE5C = 1;
                   if (b!=0xDE) bDEC0DE5C=0;
    b = fgetc(fp); if (b!=0xC0) bDEC0DE5C=0;
    b = fgetc(fp); if (b!=0xDE) bDEC0DE5C=0;
    b = fgetc(fp); if (b!=0x5C) bDEC0DE5C=0;
   
    if (!bDEC0DE5C) {
      
      /* remember that this happened */
      eStatus = eStatus | IDA_STATUS_DEC0DE5C_NOT_FOUND; 
      if (bVerbose) printf("[%s] Missed an expected 0xDEC0DE5C; nFrames is still %ld\n",ME,*nFrames); 

    } else {

      bBadFrame = 0; /* assume this is a good frame, for now */
      (*nFrames)++; /* track total number of frames seen */
      if (bVerbose) { printf("[%s] Saw 0xDEC0DE5C; nFrames is now %ld\n",ME,*nFrames); }

      /* Skip next 8 bytes (ID, FrameCount, SecondsCount) */
      for (k=0;k<8;k++) b = fgetc(fp);

      /* Next 2 bytes are TB?_ID for TBW and TBN, and Decimation Factor for DRX */
      b1 = fgetc(fp);
      b2 = fgetc(fp);
      if (bVerbose) { printf("[%s] TB?_ID/DF bytes: 0x%02x%02x\n",ME,b1,b2); }
      bTBWBit  = ( b1 & 0x80 ) >> 7; /* =1 for TBW, =0 for TBN, =X for DRX */
      *b4bit   = ( b1 & 0x40 ) >> 6; /* =1 for 4-bit TBW, =0 for 12-bit TBW, =X for TBN or DRX */
      if (bVerbose) printf("[%s] In ida_parse_file(), *b4bit = %d\n",ME,*b4bit);

      /* Interpretation of these two bytes is mode-dependent */
      switch (eMode) {

        case IDA_MODE_TBW:
          if (!bTBWBit) {
            printf("[%s] In ida_parse_file(), eMode=IDA_MODE_TBW but bTBWBit=0\n",ME);
            eStatus = eStatus | IDA_STATUS_WRONG_TB;
            { /* crash out: */
              bBadFrame = 1; /* flag as a bad frame */
              (*nFramesBad)++; 
              if ((*iFirstBadFrame)<1) *iFirstBadFrame = *nFrames;
              return eStatus; 
              }
            } 
          if ( b4bit0 < 0 ) { b4bit0 = *b4bit; } /* set this the first time we see it */
          if ( *b4bit != b4bit0 ) {              /* this has changed mid-file... */
            if (bVerbose) printf("[%s] In ida_parse_file(), eMode=IDA_MODE_TBW and *b4bit changed to %d\n",ME,*b4bit);
            if (bVerbose) printf("[%s] In ida_parse_file(), ...iFrame = %ld\n",ME,*nFrames);
            eStatus = eStatus | IDA_STATUS_4BIT_CHANGED;
            bBadFrame = 1; /* flag as a bad frame */
            //return eStatus;
            }
          /* parsing out stand and pol IDs below, as this part is the same between TBW and TBN */
          break;

        case IDA_MODE_TBN:
          if (bTBWBit) {
            printf("[%s] In ida_parse_file(), eMode=IDA_MODE_TBN but bTBWBit=1\n",ME);
            eStatus = eStatus | IDA_STATUS_WRONG_TB;
            { /* crash out: */
              bBadFrame = 1; /* flag as a bad frame */
              (*nFramesBad)++; 
              if ((*iFirstBadFrame)<1) *iFirstBadFrame = *nFrames;
              return eStatus; 
              }
            } 
          /* nothing analogous to the 4/12-bit-mode bit for TBN, so nothing else to do */
          /* parsing out stand and pol IDs below, as this part is the same between TBW and TBN */
          break;

        case IDA_MODE_DRX: 
          printf("[%s] In ida_parse_file(), eMode=IDA_MODE_DRX: Not implemented\n",ME);
          eStatus = eStatus | IDA_STATUS_INVALID_MODE;
          bBadFrame = 1; /* flag as a bad frame */
          //return eStatus;
          break;

        default:
          printf("[%s] In ida_parse_file(), eMode=%d is not valid\n",ME,eMode);
          eStatus = eStatus | IDA_STATUS_INVALID_MODE;
          bBadFrame = 1; /* flag as a bad frame */
          //return eStatus;
          break;

        } /* switch (eMode) */

      if (!bBadFrame) {

        /* For TBW, parse out stand ID and do some accounting */
        if (eMode==IDA_MODE_TBW) {
          bb = ( b1 << 8 ) + ( b2 & 0xFF ); /* concatenating the two bytes to get Stand ID*/
          bb = bb & 0x3FFF;                 /* zeroing 2 high order bits */ 
          if (bVerbose) printf("[%s] In ida_parse_file(), TBW Stand ID is %d\n",ME,bb);
          if ( bb > *iStandMax ) *iStandMax = bb; /* if this is the highest-numbered stand we've seen, remember that */ 
          if (!bStand[bb]) { /* if not done already, open an output file for this stand */
            bStand[bb] = 1;  /* remember that we've already opened an output file for this stand */
            sprintf(outfilename,"ida%03d.dat",2*bb-1);
            fpo[2*bb-1] = fopen(outfilename,"wb");     /* one for x pol... */
            sprintf(outfilename,"ida%03d.dat",2*bb  );
            fpo[2*bb  ] = fopen(outfilename,"wb");     /* ... and one for y pol */
            }
          iStand = bb;      /* remember that this is where we are */
          iFrame[2*iStand-1]++; /* remember how many frames for this stand we've seen */
          iFrame[2*iStand  ]++;  
          }

        /* For TBN, parse out channel (2*stand+pol) ID and do some accounting */
        if (eMode==IDA_MODE_TBN) {
          bb = ( b1 << 8 ) + ( b2 & 0xFF ); /* concatenating the two bytes to get Channel ID */
          bb = bb & 0x3FFF;                 /* zeroing 2 high order bits */ 
          if (bVerbose) printf("[%s] In ida_parse_file(), TBN Channel ID is %d\n",ME,bb);
          k = (bb+1)/2; /* convert channel ID to stand ID */
          if ( k > *iStandMax ) *iStandMax = k; /* if this is the highest-numbered stand we've seen, remember that */ 
          if (!bStand[k]) { /* if not done already, open an output file for this stand */
            bStand[k] = 1;  /* remember that we've already opened an output file for this stand */
            sprintf(outfilename,"ida%03d.dat",2*k-1);
            fpo[2*k-1] = fopen(outfilename,"wb");     /* one for x pol... */
            sprintf(outfilename,"ida%03d.dat",2*k  );
            fpo[2*k  ] = fopen(outfilename,"wb");     /* ... and one for y pol */
            }
          iStand = k;   /* remember that this is where we are */
          iChan = bb;   
          iFrame[bb]++; /* remember how many frames for this channel we've seen */
          }

        } else { /* if (!bBadFrame) */  

          (*nFramesBad)++;
          if ((*iFirstBadFrame)<1) *iFirstBadFrame = *nFrames;

        }

      /* Next 2 bytes are unassigned for TBW and TBN, and Time Offset for DRX */
      b1 = fgetc(fp);
      b2 = fgetc(fp);
      if (bVerbose) { printf("[%s] In ida_parse_file(), Unassigned/TimeOffset bytes: 0x%02x%02x\n",ME,b1,b2); }

      /* This is where you will deal with TimeOffset, for DRX */

      /* Next 8 bytes are TimeTag */
      i8u.b[7] = (char) fgetc(fp); //printf("TimeTag = 0x%02x",b);
      i8u.b[6] = (char) fgetc(fp); //printf("%02x",b);
      i8u.b[5] = (char) fgetc(fp); //printf("%02x",b);
      i8u.b[4] = (char) fgetc(fp); //printf("%02x",b);
      i8u.b[3] = (char) fgetc(fp); //printf("%02x",b);
      i8u.b[2] = (char) fgetc(fp); //printf("%02x",b);
      i8u.b[1] = (char) fgetc(fp); //printf("%02x",b);
      i8u.b[0] = (char) fgetc(fp); //printf("%02x\n",b);
      //i8u.i = 0x8000000000000000 ^ i8u.i; /* flip high bit */
      if (bVerbose) printf("[%s] In ida_parse_file(), time tag is %lu (0x%lx)\n",ME,i8u.i,i8u.i);
      if (ttt0==0) { 
        ttt0 = i8u.i;
        *time0 = i8u.i;  
        } /* first time tag we've seen; normalize rest relative to this */
      switch (eMode) {
        case IDA_MODE_TBW:
          ttt[2*iStand-1][iFrame[2*iStand-1]] = i8u.i - ttt0;
          ttt[2*iStand  ][iFrame[2*iStand  ]] = i8u.i - ttt0;
          break;
        case IDA_MODE_TBN:
          ttt[iChan     ][iFrame[iChan     ]] = i8u.i - ttt0;
          break;
        case IDA_MODE_DRX:

          break;
        }

      /* Data packaging and format is mode-dependent */
      switch (eMode) {

        case IDA_MODE_TBW:
          /* X0, Y0 come in 3 byte groups */
          if (!(*b4bit)) { /* this is 12-bit data */
              //  12 bit samples:      X0(11:4), [X0(3:0) Y0(11:8)], Y0(7:0)
              for (k=0;k<400;k++) {
                x0 = 0;
                y0 = 0;
                b = fgetc(fp); x0 = ( b << 4 );
                b = fgetc(fp); x0 = x0 | ( ( b & 0x00F0 ) >> 4 ); y0 = ( b << 8 ) & 0x0F00;
                b = fgetc(fp);                                    y0 = y0 | ( b & 0x00FF );   
                x0 = x0 ^ 0x0800; /* flip high bit */        
                y0 = y0 ^ 0x0800; /* flip high bit */ 
                //printf("%d %d\n",x0,y0);
                if (!bBadFrame) {
                  fwrite(&x0,sizeof(short int),1,fpo[2*iStand-1]);
                  fwrite(&y0,sizeof(short int),1,fpo[2*iStand  ]);
                  }
                }
            } else { /* this is 4-bit data */
              //  4 bit samples:       [X0(3:0) Y0(3:0)], [X1(3:0) Y1(3:0)], [X2(3:0) Y2(3:0)]
              for (k=0;k<600;k++) {
                c1 = fgetc(fp); 
                c2 = fgetc(fp);
                c = ( c1 & 0xF0 ) | ( c2 >>   4 ); /* pack two x's together */
                  c = c ^ 0x88;                    /* flip high bits */
                  if (!bBadFrame) {
                    fwrite(&c,sizeof(char),1,fpo[2*iStand-1]);
                    //printf("* c1=0x%02hhx, c2=0x%02hhx => 0x%02hhx ",c1,c2,c);
                    }
                c = ( c1 <<   4 ) | ( c2 & 0x0F ); /* pack two y's together */
                  c = c ^ 0x88;                    /* flip high bits */
                  if (!bBadFrame) {
                    fwrite(&c,sizeof(char),1,fpo[2*iStand  ]);   
                    //printf("0x%02hhx\n",c);
                    }
                }
            } /* if (!(*b4bit)) */
          break;

        case IDA_MODE_TBN:
          for (k=0;k<512;k++) {
            b1 = fgetc(fp);                     /* this is I */ 
            b2 = fgetc(fp);                     /* this is Q */
            x0 = ( b1 << 8 ) | ( b2 & 0x00FF ); /* pack them into a short int */
            x0 = x0 ^ 0x8080;                   /* flip high bit of I and Q*/                      
            if (!bBadFrame) fwrite(&x0,sizeof(short int),1,fpo[iChan]);
            }
          break;

        case IDA_MODE_DRX: 

          break;

        default:
          /* shouldn't be here if previous switch() is working properly */
          break;

        } /* switch (eMode) */

      } /* if (bDEC0DE5C) */

    } /* while ( (b = fgetc(fp)) != EOF )  */

  /* close files */
  fclose(fp);                                       /* input file */
  for (k=1;k<=(*iStandMax);k++) { 
    fclose(fpo[2*k-1]); 
    fclose(fpo[2*k  ]); 
    } 

  /* Figure out the highest number of frames for any channel */
  iFrameMax = 0;
  for (k=1;k<=(2*(*iStandMax));k++) {
    if (iFrame[k]>iFrameMax) iFrameMax=iFrame[k];
    }

  /* Now, write out time tags */
  fp = fopen("ida_tt.dat","w");
  for (l=1;l<=iFrameMax;l++) {
    for ( k=1; k<=(2*(*iStandMax)); k++ ) {
      fprintf(fp,"%12ld ",ttt[k][l]);
      }
    fprintf(fp,"\n");
    }
  fclose(fp);

  /* If *b4bit changed, change it back */
  *b4bit = b4bit0;

  return eStatus;
  }

/****************************************************************************/
/*** ida_sfile_import() *****************************************************/
/****************************************************************************/
        
int ida_sfile_import( 
    int iStand,    /* IN: Stand ID */
    int iPol,      /* IN: 1 means use x-pol, 2 means use y-pol */
    int eMode,     /* IN: one of IDA_MODE_* */
    int b4bit,     /* IN: 1 means this is 4-bit data */
    short int *ti, /* OUT: For TBW, this is the data; For TBN/DRX, this is the real part */
    short int *tq, /* OUT: For TBW, this is nothing;  For TBN/DRX, this is the imag part */
    long int *n    /* OUT: number of samples read */ 
    ) {
  /* Reads sensor file ida###.dat where ### = 2*(iStand-1)+iPol */
  /* Returns data for the specified polarization in tx[] */
  /* Returns 0 if read failed; 1 otherwise */
  
  int iSensor;
  char filename[IDA_MAX_FILENAME_LENGTH];
  FILE *fp;
  long int k=0;
  short int x12;
  short int y12;
  char c; 

  iSensor = 2*(iStand-1)+iPol;
  printf("[%s] In ida_sfile_import(), iStand=%d, iPol=%d -> iSensor=%d\n",ME,iStand,iPol,iSensor);

  sprintf(filename,"ida%03d.dat",iSensor);
  if (!(fp = fopen(filename,"rb"))) { return 0; } 

  switch (eMode) {

    case IDA_MODE_TBW:
      if (!b4bit) { /* 12 bit data */
          while ( fread(&x12,sizeof(short int),1,fp) > 0 ) {
            k++;
            ti[k-1] = x12; 
            tq[k-1] = 0; 
            }
        } else { /* 4 bit data */
          while ( fread(&c,sizeof(char),1,fp) > 0 ) {
            k += 2;
            ti[k-2] = 0x000F & ( (short int) ( c >> 4   ) );
            tq[k-2] = 0;
            ti[k-1] = 0x000F & ( (short int) ( c & 0x0F ) );
            tq[k-1] = 0;    
            }
        }
      break;

    case IDA_MODE_TBN:
      while ( fread(&x12,sizeof(short int),1,fp) > 0 ) {
        k++;
        ti[k-1] = 0x00FF & ( x12 >> 8   ); 
        tq[k-1] = 0x00FF & ( x12 & 0xFF ); 
        }
      break;

    case IDA_MODE_DRX:
    default:
      printf("[%s] In ida_sfile_import(), invalid/unimplemented eMode = %d\n",ME,eMode);
      return 0;
      break;

    } /* switch (eMode) */

  //printf("%d %ld\n",feof(fp),k);
  //printf("tx[%ld] = %hd, x12 = %hx\n",k-1,tx[k-1],x12);
  
  fclose(fp);
  *n = k;     /* remember number of samples */
  return 1;
  }

//==================================================================================
//=== HISTORY ======================================================================
//==================================================================================
// ida.h: S.W. Ellingson, Virginia Tech, 2010 Oct 05
//   .1: First version; adapted from mtbw.h in MTBW v.0.3
//       Separating output files into separate polarizations
// mtbw.h: S.W. Ellingson, Virginia Tech, 2010 Oct 05
//   .1: Time tag now interpreted as a unsigned long int
// mtbw.h: S.W. Ellingson, Virginia Tech, 2010 Sep 30
//   .1: added support for -v (verbose) option 
//       fixed TBW_ID->StandID bug (was assuming StandID was 1 byte before)
// mtbw.h: S.W. Ellingson, Virginia Tech, 2010 Sep 23
//   .1: initial version


//==================================================================================
//=== BELOW THIS LINE IS SCRATCH ===================================================
//==================================================================================
