// idav.c: S.W. Ellingson, Virginia Tech, 2010 Oct 05
// ---
// COMPILE: See Makefile
// ---
// COMMAND LINE SYNTAX, INPUT, OUTPUT: 
//   See help message (search for "Help Message follows:" in source code below)
// ---
// REQUIRES:
// gnuplot_i (http://ndevilla.free.fr/gnuplot/gnuplot_i/index.html)
// FFTW3
// ---
// TBW data visualization.  Requires output from ida
// See end of this file for history.

#include <stdio.h>
#include <math.h>
#include <fftw3.h>
#include "gnuplot_i.h"

#define MY_NAME "ida (v.20101005.1)"
#define ME "17" /* listed in LWA_MCS.h */

#define IDAV_N_SAMPLES_MAX  36000000
#define IDAV_FS_TBW        196000000         /* TBW samples/s */
#define IDAV_FS_TBN           100000         /* TBN samples/s */
#define IDAV_NSB_MAX 262144
#define IDAV_PI 3.1415926

/* Modes */
#define IDA_MODE_TBW 1
#define IDA_MODE_TBN 2
#define IDA_MODE_DRX 3

/* Values of eWindow */ 
#define IDAV_WINDOW_NONE       0  /* no window */
#define IDAV_WINDOW_TRIANGULAR 1  /* Triangular window */
#define IDAV_WINDOW_BLACKMAN   2  /* Blackman window */
#define IDAV_WINDOW_NUMBER     2  /* number of windows defined; 1..IDAV_WINDOW_NUMBER */

static short int ti[IDAV_N_SAMPLES_MAX]; /* data read from ida###.dat (could be huge) */
static short int tq[IDAV_N_SAMPLES_MAX]; /* data read from ida###.dat (could be huge) */
static float     ax[IDAV_N_SAMPLES_MAX];     /* time-averaged data */

static float S[IDAV_NSB_MAX]; /* last spectrum */
static float P[IDAV_NSB_MAX]; /* integrated spectrum */

static float w[IDAV_NSB_MAX];   /* window */

int main( int narg, char *argv[] ) {

  int iarg;
  int bDone;

  int n=1;              /* sensor to look at */
  int eMode=IDA_MODE_TBW; /* <mode>; set in command line */
  int b4bit=0;          /* "1" means 4-bit data; "0" means 12-bit data */
  long int iBegin=1;    /* sample to begin at */ 
  long int nLength=301; /* number of samples. 0 means "to end of file" */
  int bDoWaveform=1;    /* =1 means show time-domain waveform */
  int bDoAvePower=0;    /* =1 means show time-averaged waveform */
  int bDoAveSpectrum=0; /* =1 means show averaged spectrum */
  long int nsb=1024;    /* number of samples per block = FFT length = time-domain averaging length */
  int bRemoveBias=0;    /* =1 calculate and remove zero-freq bias, block-by-block */
  int bdB=0;            /* =1 plot spectrum in dB */
  int eWindow=0;        /* >0 means apply window before FFT */
  int bPlot=1;          /* =1 means plot and pause; =0 means don't */

  FILE *fp;
  char filename[256];

  int iStand;
  int iPol;

  long int k;
  short int x12, y12;   /* used to read 12-bit data */
  char c;               /* used to read 4-bit data */

  long int u;
  long int q;
  long int r;

  float sum,sumi,sumq;

  /* FFTW variables */
  fftw_complex *in, *out;
  fftw_plan p;

  long int l;

  float df,f1,f2;
  char xlabel[256];
  char ylabel[256];
  char plot2[256];
  float it,rbw;
  float dt;
  float y2;

  /* Gnuplot variables */
  gnuplot_ctrl *h1;
  gnuplot_ctrl *h2;
  gnuplot_ctrl *h3;

  /*==================*/
  /*=== Initialize ===*/
  /*==================*/
    
  /* First, announce thyself */
  printf("[%s] I am %s\n",ME,MY_NAME);
  printf("[%s] This software uses the gnuplot_i library written by N. Devillard\n",ME);

  /* process command line arguments */
  for ( iarg=2; iarg<=narg; iarg++) {

    bDone = 0;

    /* sense & process "-h" */
    if (!strncmp(argv[iarg-1],"-h",2)) {
      bDone = 1;
      /* Help Message follows: */
      printf("COMMAND LINE:\n");
      printf("  idav -h -n<sensor> -m<mode> -4 -b<sample> -l<samples> -x -a -s -t<nsb> -o -d -w<#> -p\n");
      printf("INPUT:\n");
      printf("  -h means show help, then exit.\n");
      printf("  -n<sensor> means look at <sensor> (DEFAULT: -n1).\n");
      printf("    1 & 2 correspond to Stand 1; 3 & 4 correspond to Stand 2; and so on\n");
      printf("  -m specifies mode.  Options for <mode> are:\n");
      printf("     w (as in '-mw') means assume TBW-mode data. [DEFAULT]\n");
      printf("     n (as in '-mn') means assume TBN-mode data.\n");
      printf("     d (as in '-md') means assume DRX-mode data.\n");
      printf("  -4 means assume 4-bit data for TBW mode [DEFAULT: assume 12-bit data].\n");
      printf("  -b<sample> means begin at <sample> (DEFAULT: -b1).\n");
      printf("  -l<samples> means use <samples> samples (DEFAULT: -l301).\n");
      printf("  -x means DON'T show time-domain waveform (DEFAULT: show it).\n");
      printf("  -a means show averaged power (DEFAULT: don't).\n");
      printf("  -s means show averaged spectrum (DEFAULT: don't).\n");
      printf("  -t<nsb> means use <samples> samples per block (DEFAULT: -t1024).\n");
      printf("    This is averaging time for averaged-power output, and\n");
      printf("    FFT length for averaged-spectrum output\n");
      printf("  -o means remove zero-frequency bias before FFT (DEFAULT: don't).\n");
      printf("  -d means plot spectrum in dB (DEFAULT: linear units).\n");
      printf("  -w<#> means apply window before FFT (DEFAULT: don't).\n");    
      printf("    <#>=1 (or omitted) means triangular.\n"); 
      printf("    <#>=2 means Blackman (alpha=0.16).\n"); 
      printf("  -p means don't display plots (DEFAULT: display plots and pause).\n");  
      printf("    (Useful if only the output files are of interest.)\n"); 
      printf(" OUTPUT FILES:\n");
      printf("   idav_t.dat (waveform data), unless -x is used.\n");
      printf("   idav_a.dat (averaged power), if -a is used.\n");
      printf("   idav_s.dat (averaged spectrum), if -s is used.\n");
      printf(" ADDITIONAL INFO:\n");
      printf("   Plots are piped to Gnuplot, unless -p is used.\n");
      printf("   Plot legends are interpreted as follows: Example: '6(3/2)'\n");
      printf("     means 6th sensor, which TBW indicated as Stand 3, Pol 2 (of 2)\n");
      return;
      } 

    /* sense & process "-n" */
    if (!strncmp(argv[iarg-1],"-n",2)) {
      bDone = 1; 
      if (strlen(argv[iarg-1])<3) {
        printf("[%s] FATAL: Can't parse parameter of recognized switch <%s>\n",ME,argv[iarg-1]);
        return;
        }
      sscanf( &(argv[iarg-1][2]), "%d", &n ); 
      if ( (n<1) || (n>520) ) {
        printf("[%s] FATAL: Invalid sensor index: %d\n",ME,n);
        return;
        }
      } 

    /* sense & process "-m" */
    if (!strncmp(argv[iarg-1],"-m",2)) {
      bDone = 1;
      if (strlen(argv[iarg-1])<3) {
        printf("[%s] FATAL: Can't parse parameter of recognized switch '%s'\n",ME,argv[iarg-1]);
        return;
        }
      switch (argv[iarg-1][2]) {
        case 'w': eMode = IDA_MODE_TBW; break;
        case 'n': eMode = IDA_MODE_TBN; break;
        case 'd': eMode = IDA_MODE_DRX;
          printf("[%s] FATAL: Concerning switch '%s': DRX mode not implemented.\n",ME,argv[iarg-1]);
          return;
          break;
        default:
          printf("[%s] FATAL: Can't parse parameter of recognized switch '%s'\n",ME,argv[iarg-1]);
          return;
          break;    
        } 
      } /* if (!strncmp(argv[iarg-1],"-m",2))  */

    /* sense & process "-4" */
    if (!strncmp(argv[iarg-1],"-4",2)) {
      bDone = 1;
      b4bit = 1;
      }

    /* sense & process "-b" */
    if (!strncmp(argv[iarg-1],"-b",2)) {
      bDone = 1;   
      if (strlen(argv[iarg-1])<3) {
        printf("[%s] FATAL: Can't parse parameter of recognized switch <%s>\n",ME,argv[iarg-1]);
        return;
        }
      sscanf( &(argv[iarg-1][2]), "%ld", &iBegin ); 
      } 

    /* sense & process "-l" */
    if (!strncmp(argv[iarg-1],"-l",2)) {
      bDone = 1;   
      if (strlen(argv[iarg-1])<3) {
        printf("[%s] FATAL: Can't parse parameter of recognized switch <%s>\n",ME,argv[iarg-1]);
        return;
        }
      sscanf( &(argv[iarg-1][2]), "%ld", &nLength ); 
      } 

    /* sense & process "-x" */
    if (!strncmp(argv[iarg-1],"-x",2)) {
      bDone = 1;
      bDoWaveform = 0;
      }

    /* sense & process "-a" */
    if (!strncmp(argv[iarg-1],"-a",2)) {
      bDone = 1;
      bDoAvePower = 1;
      }

    /* sense & process "-s" */
    if (!strncmp(argv[iarg-1],"-s",2)) {
      bDone = 1;
      bDoAveSpectrum = 1;
      }

    /* sense & process "-t" */
    if (!strncmp(argv[iarg-1],"-t",2)) {
      bDone = 1;   
      if (strlen(argv[iarg-1])<3) {
        printf("[%s] FATAL: Can't parse parameter of recognized switch <%s>\n",ME,argv[iarg-1]);
        return;
        }
      sscanf( &(argv[iarg-1][2]), "%ld", &nsb ); 
      }

    /* sense & process "-o" */
    if (!strncmp(argv[iarg-1],"-o",2)) {
      bDone = 1;
      bRemoveBias = 1;
      }

    /* sense & process "-d" */
    if (!strncmp(argv[iarg-1],"-d",2)) {
      bDone = 1;
      bdB = 1;
      }

    /* sense & process "-w" */
    if (!strncmp(argv[iarg-1],"-w",2)) {
      bDone = 1;   
      if (strlen(argv[iarg-1])<3) {
          eWindow = IDAV_WINDOW_TRIANGULAR; /* triangular */
        } else {
          sscanf( &(argv[iarg-1][2]), "%d", &eWindow ); 
          if ( (eWindow<0) || (eWindow>IDAV_WINDOW_NUMBER) ) {
           printf("[%s] FATAL: In switch <%s>, %d not allowed.\n",ME,argv[iarg-1],eWindow);
           return;
           }
        }
      }

    /* sense & process "-p" */
    if (!strncmp(argv[iarg-1],"-p",2)) {
      bDone = 1;
      bPlot = 0;
      }

    if (!bDone) {
      printf("[%s] FATAL: Can't parse switch <%s>\n",ME,argv[iarg-1]);
      return;
      }
      
    } /* for (iarg */

  printf("[%s] PARAMETER: n = %d\n",ME,n);
  printf("[%s] PARAMETER: eMode = %d\n",ME,eMode);
  printf("[%s] PARAMETER: b4bit = %d\n",ME,b4bit);
  printf("[%s] PARAMETER: iBegin = %ld\n",ME,iBegin);
  printf("[%s] PARAMETER: nLength = %ld\n",ME,nLength);
  printf("[%s] PARAMETER: bDoWaveform = %d\n",ME,bDoWaveform);
  printf("[%s] PARAMETER: bDoAvePower = %d\n",ME,bDoAvePower);
  printf("[%s] PARAMETER: bDoAveSpectrum = %d\n",ME,bDoAveSpectrum);
  printf("[%s] PARAMETER: nsb = %ld\n",ME,nsb);
  printf("[%s] PARAMETER: bRemoveBias = %d\n",ME,bRemoveBias);
  printf("[%s] PARAMETER: bdB = %d (1=plot in dB; 0=plot in linear units)\n",ME,bdB);
  printf("[%s] PARAMETER: eWindow = %d (0=none, 1=Triangular, 2=Blackman)\n",ME,eWindow);
  printf("[%s] PARAMETER: bPlot = %d\n",ME,bPlot);
  //return;

  if ( ( bDoAvePower || bDoAveSpectrum) && ( nsb>nLength ) ) {
    printf("[%s] FATAL: nsb > nLength not allowed if -a or -s are set.\n",ME);
    return;
    }
  if ( nsb>IDAV_NSB_MAX ) {
    printf("[%s] FATAL: nsb > %d not allowed.\n",ME,IDAV_NSB_MAX);
    return;
    }

  /* Identify stand and pol for plot legends */
  iStand = (n+1)/2;
  iPol = n - 2*(iStand-1);
  //printf("[%s] %d %d %d\n",ME,n,iStand,iPol);
  //return;

  /* === Getting time domain data from file === */
  /* Data ends up as ti[]+jtq[], which goes 0..k-1 */

  sprintf(filename,"ida%03d.dat",n);
  if (!(fp = fopen(filename,"rb"))) { 
    printf("[%s] FATAL: Can't open <%s>.  (Did you forget to use ida first?)\n",ME,filename); 
    return;
    } 

  k = 0;
  switch (eMode) {
    case IDA_MODE_TBW:
      if (!b4bit) { /* 12 bit data */
          while ( fread(&x12,sizeof(short int),1,fp) > 0 ) {
            k++;
            ti[k-1] = x12; 
            tq[k-1] = 0; 
            }
        } else { /* 4 bit data */
          while ( fread(&c,sizeof(char),1,fp) > 0 ) {
            k += 2;
            ti[k-2] = 0x000F & ( (short int) ( c >> 4   ) );
            tq[k-2] = 0;
            ti[k-1] = 0x000F & ( (short int) ( c & 0x0F ) );
            tq[k-1] = 0;    
            }
        }
      break;

    case IDA_MODE_TBN:
      while ( fread(&x12,sizeof(short int),1,fp) > 0 ) {
        k++;
        ti[k-1] = 0x00FF & ( x12 >> 8   ); 
        tq[k-1] = 0x00FF & ( x12 & 0xFF ); 
        }
      break;

    case IDA_MODE_DRX:
      printf("[%s] FATAL: Invalid/unimplemented eMode=%d\n",ME,eMode); 
      return;
      break;

    } /* switch (eMode) */

  printf("[%s] Read %ld samples from %s.\n",ME,k,filename);
  
  fclose(fp);

  /* ========================================== */
  /* === Do display of time domain waveform === */
  /* ========================================== */

  if (bDoWaveform) {

    switch (eMode) {
      case IDA_MODE_TBW:
        if (b4bit) { 
            y2 = 15; 
          } else { 
            y2 = 4096;
          }
        dt = (1.0e+9)/IDAV_FS_TBW;  /* ns */
        sprintf(xlabel,"Time [ns]");
        sprintf(ylabel,"A/D Code");
        sprintf(plot2,"plot 'idav_t.dat' using 1:2 with linespoints title '%d(%d/%d)'",n,iStand,iPol);
        break;
      case IDA_MODE_TBN:
        y2 = 255;
        dt = (1.0e+3)/IDAV_FS_TBN;
        sprintf(xlabel,"Time [ms]"); /* ms */
        sprintf(ylabel,"TBN Output Word");
        sprintf(plot2,"plot 'idav_t.dat' using 1:2 with linespoints title '%d(%d/%d) I', 'idav_t.dat' using 1:3 with linespoints title 'Q'",n,iStand,iPol);
        break;
      case IDA_MODE_DRX:
        break;
      }

    /* Writing back out the dataset we want to show */
    fp = fopen("idav_t.dat","w");
    for ( k = (iBegin-1) ; k < (iBegin-1+nLength+1) ; k++ ) {
      fprintf(fp,"%12.3f %5d %5d\n", dt*k, ti[k], tq[k] );
      }
    fclose(fp);    
  
    /* Time domain plot */

    if (bPlot) {
      h1 = gnuplot_init();
      gnuplot_cmd(h1,"%s",plot2);
      gnuplot_cmd(h1,"set yrange [0:%f]",y2);
      gnuplot_cmd(h1,"set xlabel '%s'",xlabel);
      gnuplot_cmd(h1,"set ylabel '%s'",ylabel); 
      gnuplot_cmd(h1,"set grid; show grid");  
      gnuplot_cmd(h1,"replot");
      } /* if (bPlot) */

    } /* if (bDoWaveform) */

  /* ============================================ */
  /* === Do display of average power waveform === */
  /* ============================================ */

  if (bDoAvePower) {

    /* Doing the time averaging: */
    k = iBegin; /* sample index */
    u = 0;      /* counting samples considered */
    q = 0;      /* counting samples in a block */
    r = 0;      /* counting blocks */
    ax[r] = 0;
    while ( (u<nLength) && (k<IDAV_N_SAMPLES_MAX) ) {
      ax[r] += sqrt((float)(ti[k]*ti[k]+tq[k]*tq[k]));
      k++;
      u++;
      q++; 
      if (q==nsb) {
        ax[r] /= nsb;
        q = 0; /* reset counting of samples within a block */
        r++;   /* starting new block */
        ax[r] = 0;
        } /* if (q==nsb) */      
      } /* while (p<nLength) */
    /* r is now taken to be number number of blocks.  Technically we have */
    /* one more than this, but the last block potentially represents < nsb samples */

    /* Writing back out the dataset we want to show */
    fp = fopen("idav_a.dat","w");
    for ( k = 0 ; k < r ; k++ ) {
      /* FIXME: */
      fprintf(fp,"%10.7f %7.1f\n", ((float)k)*nsb*IDAV_FS_TBN, ax[k] );
      }
    fclose(fp);    
  
    /* Average power (over time) plot */

    if (bPlot) {
      h2 = gnuplot_init();
      gnuplot_cmd(h2,"plot 'idav_a.dat' using 1:2 with lines title '%d(%d/%d)'",n,iStand,iPol);
      switch (eMode) {
        case IDA_MODE_TBW:
          if (b4bit) {
              gnuplot_cmd(h2,"set yrange [0:15]");
            } else {
              gnuplot_cmd(h2,"set yrange [0:4095]");
            }
          break;
        case IDA_MODE_TBN:
          gnuplot_cmd(h2,"set yrange [0:255]");
          break;
        case IDA_MODE_DRX:
          break;
        }
      gnuplot_cmd(h2,"set xlabel 'Time [ms]'");
      gnuplot_cmd(h2,"set ylabel 'Average [A/D Codes]'");
      gnuplot_cmd(h2,"set title 'Int. Time = %10.3e [s]'",((float)nsb)/IDAV_FS_TBW); 
      gnuplot_cmd(h2,"set grid; show grid");  
      gnuplot_cmd(h2,"replot");
      } /* if (bPlot) */

    } /* if (bDoAvePower) */

  /* ============================================ */
  /* === Do display of averaged spectrum ======== */
  /* ============================================ */

  if (bDoAveSpectrum) {

    /* FFTW setting up */
    in = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * nsb);
    out = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * nsb);
    p = fftw_plan_dft_1d(nsb, in, out, FFTW_FORWARD, FFTW_MEASURE);

    /* initialize PSD array */
    for (q=0;q<nsb;q++) P[q]=0.0;

    /* initialize window */
    switch (eWindow) {
      case IDAV_WINDOW_TRIANGULAR:
        for (q=0;q<nsb;q++) { 
          if (q<(nsb/2)) { 
              w[q] = ((float)q)/((nsb/2)-1);
            } else {
              w[q] = 2.0 - ((float)(q-1))/((nsb/2)-1);
            }
          //printf("%ld %f\n",q,w[q]);
          }   
        break;
      case IDAV_WINDOW_BLACKMAN:
        /* Using definition from http://en.wikipedia.org/wiki/Window_function */
        { /* === BEGIN block defining Blackman window ===*/
        float alpha,a0,a1,a2;
        alpha = 0.16;
        a0 = (1-alpha)/2;
        a1 = 0.5;
        a2 = alpha/2;
        for (q=0;q<nsb;q++) { 
          w[q] = a0 - a1*cos(2*IDAV_PI*q/(nsb-1)) + a2*cos(4*IDAV_PI*q/(nsb-1));
          //printf("%ld %f\n",q,w[q]);
          }         
        } /* === END block defining Blackman window === */
        break; 
      } /* switch (eWindow) */ 
    //return;

    /* --- Main loop --- */
    k = iBegin; /* sample index */
    u = 0;      /* counting samples considered */
    r = 0;      /* counting blocks */
    ax[r] = 0;
    while ( (u<nLength) && ((k+nsb-1)<IDAV_N_SAMPLES_MAX) ) {
     
      /* load next nsb samples into FFT input array */
      q = 0;
      sumi = 0; /* for calculating bias */
      sumq = 0; /* for calculating bias */
      while (q<nsb) {
        in[q][0] = (float) ti[k];
        in[q][1] = (float) tq[k];
        out[q][0] = 0.0; /* clearing output array, just in case */
        out[q][1] = 0.0;
        sumi += in[q][0];
        sumq += in[q][1];
        k++;
        u++;
        q++; 
        } 
         
      /* remove bias if requested */
      sumi /= nsb;
      sumq /= nsb;
      if (bRemoveBias) {
        for (q=0;q<nsb;q++) {
          in[q][0] -= sumi;
          in[q][1] -= sumq;
          }
        }
      //printf("%f %f\n",sum ,((float)sum)/nsb); return;

      /* windowing */
      if (eWindow>0) {
        for (q=0;q<nsb;q++) {
          //printf("%ld %f %f ",q,w[q],in[q][0]);
          in[q][0] *= w[q];
          in[q][1] *= w[q];
          //printf("%f\n",in[q][0]);
          }
        } /* if (eWindow>0) */
      //return; 
 
      /* do that crazy FFT thing */	
      fftw_execute(p); 

      /* average output with previous FFT squared-magnitudes */
      /* calc S and P */
      for (q=0;q<nsb;q++) {
        S[q] = ((float)( ( out[q][0] * out[q][0] ) + ( out[q][1] * out[q][1] ) )) / nsb;
        P[q] += S[q];
        }    
  
      /* update number of blocks */
      r++; 

      } /* while ( (u<nLength) && ((k+nsb-1)<IDAV_N_SAMPLES_MAX) ) */

    /* shut down FFTW */
    fftw_destroy_plan(p);
    fftw_free(in); 
    fftw_free(out);

    /* complete calculation of average */
    for (q=0;q<nsb;q++) P[q] /= r;

    /* frequency bin width */
    switch (eMode) {
      case IDA_MODE_TBW: 
        df = (((float)IDAV_FS_TBW)/nsb)/(1.0e+6); /* MHz */
        f1 = 1.0;  /* MHz */
        f2 = 98.0; /* MHz */
        sprintf(xlabel,"Freq [MHz]");
        it  = ((float)(nsb*r))/IDAV_FS_TBW;
        rbw = ((float)IDAV_FS_TBW)/nsb;
        break; 
      case IDA_MODE_TBN: 
        df = (((float)IDAV_FS_TBN)/nsb)/(1.0e+3); /* kHz */
        f1 = -50.0; /* kHz */
        f2 = +50.0; /* kHz */
        sprintf(xlabel,"Freq [kHz]");
        it = ((float)(nsb*r))/IDAV_FS_TBN;
        rbw = ((float)IDAV_FS_TBN)/nsb;
        break; 
      case IDA_MODE_DRX: 
        df = 0.0; /* (((float)IDAV_FS_DRX)/nsb)/(1.0e+6); */ 
        break;
      }

    /* Writing back out the dataset we want to show */
    fp = fopen("idav_s.dat","w");
    if (eMode!=IDA_MODE_TBW) { /* write out the negative-frequency part of the spectrum */
      for ( q = (nsb/2) ; q < nsb; q++ ) {
        if (bdB) {
            fprintf(fp,"%10.6f %11.1f\n", (q-nsb)*df, 10.0*log10(P[q]) );
          } else {
            fprintf(fp,"%10.6f %11.1f\n", (q-nsb)*df, P[q] );
          }
        }
      }
    for ( q = 0 ; q < (nsb/2) ; q++ ) {
      if (bdB) {
          fprintf(fp,"%10.6f %11.1f\n", q*df, 10.0*log10(P[q]) );
        } else {
          fprintf(fp,"%10.6f %11.1f\n", q*df, P[q] );
        }
      }
    fclose(fp);    
  
    /* plot */
    if (bPlot) {
      h3 = gnuplot_init();
      gnuplot_cmd(h3,"plot 'idav_s.dat' using 1:2 with lines title '%d(%d/%d)'",n,iStand,iPol);
      gnuplot_cmd(h3,"set xlabel '%s'",xlabel);
      gnuplot_cmd(h3,"set xrange [%f:%f]",f1,f2);
      if (bdB) {
          gnuplot_cmd(h3,"set ylabel 'PSD [dB(/RBW)]'");
        } else { 
          gnuplot_cmd(h3,"set ylabel 'PSD [/RBW]'");
        }
      gnuplot_cmd(h3,"set title 'Int. Time = %10.3e [s]. RBW = %10.3e [Hz]'",it,rbw);
      gnuplot_cmd(h3,"set grid; show grid");  
      gnuplot_cmd(h3,"replot");
      } /* if (bPlot) */

    } /* if (bDoAvePower) */

  /* ========================================== */
  /* === Clean up ============================= */
  /* ========================================== */

  if (bPlot) {
    if ( bDoWaveform || bDoAvePower || bDoAveSpectrum ) {
      printf("Hit ENTER to continue..."); getchar();
      }
    if (bDoWaveform)    gnuplot_close(h1);
    if (bDoAvePower)    gnuplot_close(h2);
    if (bDoAveSpectrum) gnuplot_close(h3);
    } /* if (bPlot) */

  return 0;
  } /* main() */

//==================================================================================
//=== HISTORY ======================================================================
//==================================================================================
// idav.c: S.W. Ellingson, Virginia Tech, 2010 Oct 05
//   .1: initial version, adapted from mtbwv.c from MTBW v.0.3
// mtbwv.c: S.W. Ellingson, Virginia Tech, 2010 Oct 04
//   .1: fixed bug in computing offset subtraction
//       adding -w, -p
//       Gnuplot legend now reads n(iStand/iPol)
// mtbwv.c: S.W. Ellingson, Virginia Tech, 2010 Sep 30
//   .1: adding -d switch
// mtbwv.c: S.W. Ellingson, Virginia Tech, 2010 Sep 27
//   .1: initial version


//==================================================================================
//=== BELOW THIS LINE IS SCRATCH ===================================================
//==================================================================================
